function [Pesos_new,Error_sal]=perceptron_train(In,Sal,ErrorMax,alfa,Umbral,Pesos0,MAX)
% PERCEPTRON	Ejemplo Doctorado 2001/2002
% 
%  [Pesos_new,Errores]=perceptron_train((In,Sal,ErrorMax,alfa,Umbral,Pesos0)
%
%	In:      (NxM) Entrada para aprendizaje 
%              N: Nmero de patrones de aprendizaje
%              M: Nmero de neuronas de entrada
%	Sal:	   (NxK) Salida para aprendizaje
%              N: Nmero de patrones de aprendizaje (una salida por patrn)
%              K: Nmero de neuronas de salida
%	ErroMax:	Error mximo permitido
%   alfa:       Velocidad de aprendizaje (por defecto 1)
%	Umbral:     0,1 indica si hay o no
%	Pesos0:    ((M*K+K*Umbral)x1) Pesos iniciales
%   MAX:        Nmero mximo de iteraciones (defecto 300)
%
%
%	Pesos_new:	Pesos calculados para la red en cada iteracin.
%   Errores:    Error de la red en cada iteracin.
%               Nota: se cuenta como una iteracin la muestra de todos los
%               datos de entrada a la red.
%
%
%  Si se utiliza adaline_train((In,Sal,ErrorMax) entonces alfa es 1, no 
%  hay umbral y los pesos son aleatorios. 
%  El nmero mximo de iteraciones por defecto es de 300. 
%  Si no se introduce pesos iniciales o Pesos0=[], se calculan
%  aleatoriamente.

% APRENDIZAJE AUTOMTICO (Gonzalo Pajares y Jess M. de la Cruz, Eds.)
% Editorial: RA-MA
% Captulo 4: Redes Neuronales Artificiales
% Autores: Jos Antonio Lpez Orozco


[filas,col]=size(In);
[filas_sal,col_sal]=size(Sal);

if (nargin < 4),
   alfa=1; 
end;

if (nargin < 5),
   Umbral=0; 
end;

if (nargin<6),  
   Pesos=rand(col*col_sal+Umbral*col_sal,1);
else
   Pesos=Pesos0;
end   

if (nargin<7),
      MAX=300;
end;      

if (Umbral)
   In=[ones(filas,1),In];
end;   

cambio=1;   % Indicar si hay algn cambio en los pesos
cont=0;		% Indica el nmero de intentos;

while ( (cambio) & (cont<MAX) )
cambio=0;

for i=1:filas,
   SalD=Sal(i,:);
   
   Salida=perceptron(In(i,2:3),Pesos);
   
   Error(i)=SalD - Salida;
   if (Error(i)~=0),
      Pesos=Pesos+ alfa* Error(i) * In(i,:)';
      cambio=cambio+1;
   end;
   
   % Pesos por cada vez que se realiza una iteracin
   %Pesos_new(filas*cont+i,:)=Pesos';
   %Error_sal(filas*cont+i,:)=Error';
   
end; %for

% (error global de la red)
    %Salida=perceptron(In(:,2:3),Pesos);
    %cambio=sum(abs(Sal - Salida));

if ( cambio <= ErrorMax ),
  cambio=0; % Aunque haya cambio si el error total es menor de ErrorMax salgo
end;

cont=cont+1;
% Pesos cada vez que se muestra un conjunto de patrones
Pesos_new(cont,:)=Pesos';
Error_sal(cont,:)=cambio;

end; % while

if (cont==MAX), disp('Error el Perceptrn no converge'); end;
