% APRENDIZAJE AUTOMTICO (G. Pajares y J. M. de la Cruz, Eds.)
% Editorial: RA-MA
% Captulo 5: Mquinas de Vectores Soporte
% Autores: Jess Antonio Vega Snchez y Sebastin Dormido Canto

function  [svmC,nsvC,bC]=training(numeroclases,numero_muestras,XX,C,kernel,param);



% La funcin TRAINING realiza el entrenamiento de las mquinas utilizadas en la tcnica SVM.
%
% Los parmetros de entrada son:
%                - numeroclases: es un escalar y representa el nmero de clases entre las que se
%                                desea clasificar. Por ej. si se tiene {BOL5,ECE7,RX306} el valor de
%                                numeroclases sera 3.
%
%                - numero_muestras: es un vector donde cada elemento representa el nmero de muestras 
%                                   que hay por clase. Por ej. si se tiene BOL5_1, BOL5_2, ECE7_1, ECE7_2, ECE7_3,
%                                   y RX306_1, numero_muestras=[2 3 1]
%
%                - XX: es la matriz de los vectores de entrenamiento (se genera con el programa norm_svm). Ver 
%                      norm_svm para observar el formato.
%
%                -  C: es un vector que almacena el parmetro de regularizacin (lmite superior de los alphas).
%                      Cada elemento representa el lmite para cada clase. Ej. C=[Inf Inf 1];
%
%                - kernel: es un string donde se introduce la funcin de transformacin a utilizar.                  
%                          Los valores para kernel pueden ser: 
%                                                           'linear'  -
%                                                           'poly'    - [1,p1] p1 is degree of polynomial
%                                                           'erfb'    - [1,p1] p1 is width of rbfs (sigma)
%                                                           'rbf'     - [1,p1] p1 is width of rbfs (sigma)
%                                                           'sigmoid' - [p1,p2] p1 is scale, p2 is offset

%
%                - param: es un vector de dimensin 2 que almacena los parmetros de las funciones kernel que
%                         se pueden utilizar. Por ejemplo param=[2 1]; 
%
% Los parmetros de salida son:
%                - svmC: almacena los vectores soporte de todas las mquinas (se almacenan en filas), la penltima
%                       columna almacena el valor de la etiqueta {1,-1} para saber si pertenece a la clase k 
%                       al resto y en la ltima columna el valor de alpha
%
%                - nsvC: almacena el nmero de vectores soporte de cada una de las k mquinas
%
%                - bC: alamcena el trmino bias de las k mquinas


%Ejemplo Multiclase SVM
%Entrenar Mquina
%----------------------------------------

%clc;
%Inicializacin de variables que se necesitan en la funcin
senalC=[];
wC=[];
bC=[];
z=[];
p=0;
svmC=[];
nsvC=[];
bC=[];

  
%disp('Preparando Entrenamiento');

%Entrena una mquina contra todas
cont=1;
for k=1:numeroclases
   waux=[];
   baux=[];
   psv=[];
   svm=[];
   
  % disp(['Entrenando Maquina ',num2str(k)]);
  % disp('---------------------------------');
   
   % En Clase (vector columna) se almacenan las etiquetas {1,-1} que identifican si los vectores de entrenamiento
   % pertenecen a la clase considerada {1}  no (es decir, pertenecen al resto) {-1}
   clase=-1*ones(size(XX,1),1);
   clase(cont:cont+numero_muestras(k)-1)=1;
   
     
   cont=cont+numero_muestras(k);

     
%   disp([' 1:   Clase ',num2str(k)]);
%   disp(['-1:No Clase ',num2str(k)]);
   
  % Se almacena en c el valor del lmite de los alphas de la mquina k 
   c=C(k);
   
   % Se llama a la funcin que realiza el entrenamiento de la mquina k
   %
   % svm almacena los vectores soporte de la mquina k (se almacenan en filas), la penltima columna almacena el valor
   % de la etiqueta {1,-1} para saber si pertenece a la clase k  al resto y en la ltima columna el valor de alpha
   %
   % psv es una matriz de dos columnas, en la primera se almacenan los ndices absolutos de los vectores soporte y
   % en la segunda se almacena una etiqueta {1,-1} que identifica si el vector soporte corresponde a la clase k {1}
   %  al resto {-1}
   %
   % En b0 se almacena el valor del parmetro de bias y en nsv el nmero de vectores soporte para la mquina k
   [svm psv b0 nsv]=wsvm(XX,clase,kernel,c,param);
   
   %disp(['Se obtuvieron ',num2str(size(psv,1)),' vectores soporte de un total de ',num2str(length(clase))]);
   %disp(['Los vectores soporte de la Clase ',num2str(k),' son:']);
	%disp(psv(find(psv(:,2)==1),1));
	%disp(['Los vectores soporte que no pertenecen a la Clase ',num2str(k),' son:']);
	%disp(psv(find(psv(:,2)==-1),1));
   
   
   % En svmC se van almacenando todas las matrices de las k mquinas 
   svmC=[svmC;svm];
   
   % En nsvC se van almacenando el nmero de vectores soporte de cada una de las k mquinas 
   nsvC=[nsvC;nsv];
   
   % En bC se va almacenando el trmino bias de cada mquina
   bC(k)=b0;
  
end;

return
