function x = gsamp(mu, covar, nsamp)

%GSAMP	Sample from a Gaussian distribution.
%
%	Description
%
%	X = GSAMP(MU, COVAR, NSAMP) generates a sample of size NSAMP from a
%	D-dimensional Gaussian distribution. The Gaussian density has mean
%	vector MU and covariance matrix COVAR, and the matrix X has NSAMP
%	rows in which each row represents a D-dimensional sample vector.
%
%	See also
%	GAUSS, DEMGAUSS
%

%   APRENDIZAJE AUTOMTICO (G. Pajares y J. M. de la Cruz, Eds.)
%   Editorial: RA-MA
%   Captulo 8: Monte Carlo II
%   Autores (version actual): Eva Besada-Portas, Sergey M. Plis

%	Original version: Copyright (c) Ian T Nabney (1996-2001)
%   It is part of the netlab Matlab lib
%   http://www.mathworks.com/matlabcentral/fileexchange/2654

d = size(covar, 1);
mu=mu(:);mus=repmat(mu',nsamp,1);
[evec, eval] = eig(covar);
coeffs = randn(nsamp, d)*sqrt(eval);
x = mus + coeffs*evec';
