function outIndex = ResampleResidual(t,inIndex,q,N,unsort)

%ResampleResidual A Resampling function
%
% outIndex = ResampleResidual(t,inIndex,q,N);
%
%  Performs the resampling stage of the SIR  in order(number of samples) steps. 
%  It uses Liu's residual resampling algorithm and Niclas' magic line.
%
% t: time step
% inIndex = Input particle indices.
% q: Normalised weights.
% N: number of elements to sample
% outIndex = Resampled indices.
%
%
% As a resampling method it must contain t,indIndex,w,N variables


% APRENDIZAJE AUTOMTICO (G. Pajares y J. M. de la Cruz, Eds.)
% Editorial: RA-MA
% Captulo 8: Monte Carlo II
% Autores: Eva Besada-Portas, Sergey M. Plis
%
% Original version: Arnaud Doucet and Nando de Freitas 
% Unscented PF toolbox
% http://www.cs.ubc.ca/~nando/software.php


q=q(:);
[S,arb] = size(q);  % S = Number of particles.

% RESIDUAL RESAMPLING:
% ====================
N_babies= zeros(1,S);
% first integer part
q_res = N.*q'; %'
N_babies = fix(q_res);
% residual number of particles to sample
N_res=N-sum(N_babies);
%save tonto q_res N_res q

if (N_res~=0)
  q_res=(q_res-N_babies)/N_res;
  cumDist= cumsum(q_res);   
  % generate N_res ordered random variables uniformly distributed in [0,1]
  u = fliplr(cumprod(rand(1,N_res).^(1./(N_res:-1:1))));
  j=1;
  for i=1:N_res 
    while (u(1,i)>cumDist(1,j))
      j=j+1;
    end
    %j
    N_babies(1,j)=N_babies(1,j)+1;
  end;
end;



% COPY RESAMPLED TRAJECTORIES:  
% ============================
index=1;
for i=1:S
  if (N_babies(1,i)>0)
    for j=index:index+N_babies(1,i)-1
      outIndex(j) = inIndex(i);
    end;
  end;   
  index= index+N_babies(1,i);   
end

if unsort
    [tt,ind]=sort(rand(N,1));
    outIndex=outIndex(ind);
end












