function GACerveza(HORASC,NGENER,howoften)

% GACERVEZA Runs the GA for solving the beer problem
%
% GaCerveza(HORASC,NGENER,storehowoften)
%    HORASC: Number of hours of the whole process (such as 150)
%    NGENER: Number of generations of the genetic algorithm
%    storehowoften: steptime used to save the data in the files
%                   results_HORASC_step


% APRENDIZAJE AUTOMTICO (G. Pajares y J. M. de la Cruz, Eds.)
% Editorial: RA-MA
% Captulo 9: El Aprendizaje con Algoritmos Genticos
% Autores: Bonifacio de Andres y Toro


if ~exist('is_new')
    warning('You need to have the EVOCOM Toolbox (http://www.dacya.ucm.es/evocom) installed to run this program');
    return;
end


DIBUJAR=0;

SEED=sum(clock*100);

TGEN=1;  % 1 hora por intervalo
NPOBL=10;          
NSUBT=4;
NGENES=HORASC+1;   	%NGENES es el numero de horas + 1 
NPCRUCE=2; %antes habia 2   
%TOPE=620;    
CRUCE=0.8;
MUTA=0.008;      

HORAS=(NGENES-1)*TGEN;
Ti=10;   
Tf=16;         
Tinit=10;

SALTOSUPER=20; 
SALTO=[2.0 1.5 1.5 1.0,repmat([ 1.0 .5 .3 .2 .2 .1 .1 .1 .2 .2 .3 .5],1,ceil(NGENER/SALTOSUPER))];


%      J=	 [	EA		e		sumamulb		vdk		distancia		calor		 dcalor];
prior=       [   1  	1	   2	     		1			3					2				];
lowgoal=     [-15, 	60,   -100,		  	-.2 ,	  - Inf	,	     -Inf         ];
upgoal=	   [Inf	, Inf,   -2,	  		Inf	 ,  	Inf	,  	 	Inf      ];
escalado=20;


ind_str=is_new(1);
ind_str=is_set(ind_str,'gen_number',NGENES);
ind_str=is_set(ind_str,'fungen','gen_cerveza');
ind_str=is_set(ind_str,'paramgen',{[Ti,Tf],Tinit});
ind_str=is_set(ind_str,'funeval','gev_equal');
ind_str=is_set(ind_str,'parameval',[]);
ind_str=is_set(ind_str,'funrepair','repair_cerveza');
ind_str=is_set(ind_str,'paramrepair',{[Ti,Tf],Tinit});

obj_str=objs_new(1);
obj_str=objs_set(obj_str,'proce_cerveza',HORAS,6,1);

gas_str=gas_new(length(ind_str),sum(cell2mat(objs_fields(obj_str,3))),2);
gas_str=gas_set(gas_str,'number','popinit',NPOBL);
gas_str=gas_set(gas_str,'number','parents',NSUBT);
gas_str=gas_set(gas_str,'number','ninmigrant',2);


gas_str=gas_set(gas_str,'fitness','obj','max');
gas_str=gas_set(gas_str,'fitness','fitfun','fit_multigoalprior');
param=param_new(5);
param=param_set(param,prior,1);
param=param_set(param,lowgoal,2);
param=param_set(param,upgoal,3);
param=param_set(param,'Goldberg',4);
param=param_set(param,escalado,5);
gas_str=gas_set(gas_str,'fitness','fitparam',param);

gas_str=gas_set(gas_str,'selec','fun','sel_roulette');

gas_str=gas_set(gas_str,'cross','prob',CRUCE);
gas_str=gas_set(gas_str,'cross','pointparam',NPCRUCE);

gas_str=gas_set(gas_str,'mut','selgenesparam',MUTA);

%gas_str=gas_set(gas_str,'recombine','fun','rec_subgen');
%gas_str=gas_set(gas_str,'recombine','param',[]);

gas_str=gas_set(gas_str,'recombine','fun','rec_elitism');
gas_str=gas_set(gas_str,'recombine','param',1);

gas_str=gas_set(gas_str,'stop','param',NGENER);

gas_str=gas_set(gas_str,'change','fun','ga_mut');
param=param_new(4);
param=param_set(param,gas_get(gas_str,'mut','selgenesfun'),1);
param=param_set(param,0.08,2);
param=param_set(param,gas_get(gas_str,'mut','new_valuefun'),3);
param=param_set(param,gas_get(gas_str,'mut','new_valueparam'),4);
gas_str=gas_set(gas_str,'change','param',param);

if DIBUJAR
gas_str=gas_set(gas_str,'show','fun','show_bestn',1);
gas_str=gas_set(gas_str,'show','param',20,1);
%gas_str=gas_set(gas_str,'show','fun','show_cerveza',2);
%gas_str=gas_set(gas_str,'show','param',1,2);
end

gas_str=gas_set(gas_str,'store','fun','store_savebestn');
gas_str=gas_set(gas_str,'store','param',{1,howoften,sprintf('results_%d',HORASC)});

gas_str=gas_set(gas_str,'super','fun','super_profile');
gas_str=gas_set(gas_str,'super','param',SALTO);
gas_str=gas_set(gas_str,'super','step',SALTOSUPER);
gas_str=gas_set(gas_str,'super','howmany',1);
gas_str=gas_set(gas_str,'super','fixed',0);

[gen,chrom,value,fitness]=ga_algorithm(ind_str,gas_str,obj_str,SEED);